% Solve SIR model using KSA method
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%      Exact numerical integration of the master     %%%%%%%%%%%%
%%%%%%%%%%%        equation in some models of stochastic       %%%%%%%%%%%%
%%%%%%%%%%%                     epidemiology                   %%%%%%%%%%%%
%%%%%%%%%%%                      KSA METHOD                    %%%%%%%%%%%%
%%%%%%%%%%%              Authors: Jenkinson/Goutsias           %%%%%%%%%%%%
%%%%%%%%%%%               Last Modified: 01/13/2012            %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize simulation parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

tic;        % start Timer

tf  = 25;   % simulation time in days
tol = 1e-3; % EXPOKIT TOL parameter
L0  = 65;   % Krylov subspace dimension

display('Running KSA method. Please wait...');

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize model parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

s0 = 762;            % initial number of susceptible individuals
i0 = 1;              % initial number of infected individuals
r0 = 0;              % initial number of resistant individuals 

N = 3;               % number of species
M = 2;               % number of reactions

k1 = 0.00218;        % 1/day, S + I -> 2I
k2 = 0.44036;        % 1/day,     I -> R

K  = 763*764;        % number of states in sample space Z

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct generator matrix of Markov process
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

row   = zeros(3*K,1);
col   = zeros(3*K,1);
value = zeros(3*K,1);
count = 1;

for iPlusOne = 1:764
    for sPlusOne = 1:763
        
        %
        % find propensities
        %
        propensity(1) = 0;
        propensity(2) = 0;
        if iPlusOne < 763        
            propensity(1) = k1*(sPlusOne-1)*(iPlusOne-1);
        end
        propensity(2) = k2*(iPlusOne-1);
        
        %
        % add value to diagonal
        %
        diagVal      = (iPlusOne-1)*763+sPlusOne;
        row(count)   = diagVal;
        col(count)   = diagVal;
        value(count) = -(propensity(1)+propensity(2));
        count        = count+1;

        %
        % add propensity for reaction 1
        %

        if propensity(1) > 0
            % what is the location with one less S and one more I
            location1    = (iPlusOne)*763+(sPlusOne-1);            
            row(count)   = location1;
            col(count)   = diagVal;
            value(count) = propensity(1);
            count        = count+1;
        end

        %
        % add propensity for reaction 2
        %

        if propensity(2) > 0
            % what is the location with the same S and one less I
            location2    = (iPlusOne-2)*763+sPlusOne;            
            row(count)   = location2;
            col(count)   = diagVal;
            value(count) = propensity(2);
            count        = count+1;
        end
        
    end
end

Q = sparse(row(value~=0),col(value~=0),value(value~=0),K,K);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set initial condition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

p0       = zeros(size(Q,1),1);
iPlusOne = i0+1;
sPlusOne = s0+1;
p0((iPlusOne-1)*763+sPlusOne) = 1;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Use EXPOKIT software package
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[prob,err,hump] = mexpv(tf,Q,p0,tol,L0);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Map probability to SI plane
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

count = 1;
SIprobs = zeros(763,764);
for iPlusOne = 1:764
    for sPlusOne = 1:763
        SIprobs(sPlusOne,iPlusOne) = prob(count);
        count = count+1;
    end
end

timeElapsed = toc; % store the time spent in calculation

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute L2 error
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

load ../SIprobsGroundTruth.mat

FinalError = sqrt(sum(sum((SIprobsTruth-SIprobs).^2)));

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Save results to file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

saveResultsToFile;