% Draws random numbers from exponential distribution
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%      Exact numerical integration of the master     %%%%%%%%%%%%
%%%%%%%%%%%        equation in some models of stochastic       %%%%%%%%%%%%
%%%%%%%%%%%                     epidemiology                   %%%%%%%%%%%%
%%%%%%%%%%%              FIGURE GENERATION SOFTWARE            %%%%%%%%%%%%
%%%%%%%%%%%             Authors: Jenkinson/Goutsias            %%%%%%%%%%%%
%%%%%%%%%%%              Last Modified: 01/13/2012             %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function r = exprnd(lambda,m,n);
%EXPRND	Random matrices from exponential distribution.
%	R = EXPRND(LAMBDA) returns a matrix of random numbers chosen   
%	from the exponential distribution with parameter LAMBDA.
%	The size of R is the size of LAMBDA.
%	Alternatively, R = EXPRND(LAMBDA,M,N) returns an M by N matrix. 
 
%	EXPRND uses a simple inversion method. See Devroye, page 392.

%	References:
%	   [1]  L. Devroye, "Non-Uniform Random Variate Generation", 
%	   Springer-Verlag, 1986.

%	Copyright (c) 1993 by The MathWorks, Inc.
%	$Revision: 1.1 $  $Date: 1993/05/24 18:54:18 $

if nargin <  1, 
    error('Requires at least one input argument.'); 
end

    
if nargin == 1
    [errorcode rows columns] = rndcheck(1,1,lambda);
end

if nargin == 2
    [errorcode rows columns] = rndcheck(2,1,lambda,m);
end

if nargin == 3
    [errorcode rows columns] = rndcheck(3,1,lambda,m,n);
end

if errorcode > 0
    error('Size information is inconsistent.');
end

%Initialize r to zero.
r = zeros(rows, columns);

u = rand(rows,columns);
r = - lambda .* log(u);

% Return NaN if b is not positive.
if any(any(lambda <= 0));
    if prod(size(lambda) == 1)
        r = NaN * ones(rows,columns);
    else
        k = find(lambda <= 0);
        r(k) = NaN * ones(size(k));
    end
end